@@ -17,8 +17,9 @@ use List::UtilsBy qw(max_by);
 use JSON qw(decode_json encode_json);
 
 use App::ManiacDownloader::_SegmentTask;
+use App::ManiacDownloader::_BytesDownloaded;
 
-our $VERSION = '0.0.8';
+our $VERSION = '0.0.9';
 
 my $DEFAULT_NUM_CONNECTIONS = 4;
 my $NUM_CONN_BYTES_THRESHOLD = 4_096 * 2;
@@ -28,11 +29,10 @@ has '_ranges' => (isa => 'ArrayRef', is => 'rw');
 has '_url' => (is => 'rw');
 has '_url_basename' => (isa => 'Str', is => 'rw');
 has '_remaining_connections' => (isa => 'Int', is => 'rw');
-has ['_bytes_dled', '_bytes_dled_last_timer'] =>
-    (isa => 'Int', is => 'rw', default => sub { return 0;});
 has '_stats_timer' => (is => 'rw');
 has '_last_timer_time' => (is => 'rw', isa => 'Num');
 has '_len' => (is => 'rw', isa => 'Int');
+has '_downloaded' => (is => 'rw', isa => 'App::ManiacDownloader::_BytesDownloaded', default => sub { return App::ManiacDownloader::_BytesDownloaded->new; });
 
 sub _serialize
 {
@@ -42,7 +42,7 @@ sub _serialize
     +{
         _ranges => [map { $_->_serialize() } @{$self->_ranges}],
         _remaining_connections => $self->_remaining_connections,
-        _bytes_dled => $self->_bytes_dled,
+        _bytes_dled => $self->_downloaded->_total_downloaded,
         _len => $self->_len,
     };
 }
@@ -69,18 +69,15 @@ sub _start_connection
 
     sysseek( $r->_fh, $r->_start, SEEK_SET );
 
-    http_get $self->_url,
-    headers => { 'Range'
-        => sprintf("bytes=%d-%d", $r->_start, $r->_end-1)
-    },
-    on_body => sub {
+    # We do these to make sure the cancellation guard does not get
+    # preserved because it's in the context of the closures.
+    my $on_body = sub {
         my ($data, $hdr) = @_;
 
         my $ret = $r->_write_data(\$data);
 
-        $self->_bytes_dled(
-            $self->_bytes_dled + $ret->{num_written},
-        );
+        $self->_downloaded->_add($ret->{num_written});
+
         my $cont = $ret->{should_continue};
         if (! $cont)
         {
@@ -105,31 +102,56 @@ sub _start_connection
             }
         }
         return $cont;
+    };
+
+    my $final_cb = sub { return ; };
+
+    my $guard = http_get $self->_url,
+    headers => { 'Range'
+        => sprintf("bytes=%d-%d", $r->_start, $r->_end-1)
     },
-    sub {
-        # Do nothing.
-        return;
-    }
-    ;
+    on_body => $on_body,
+    $final_cb;
+
+    $r->_guard($guard);
+
+    $guard = '';
+
+    return;
 }
 
+my $MAX_CHECKS = 6;
+
 sub _handle_stats_timer
 {
     my ($self) = @_;
 
-    my $num_dloaded = $self->_bytes_dled - $self->_bytes_dled_last_timer;
+    my ($num_dloaded, $total_downloaded)
+        = $self->_downloaded->_flush_and_report;
+
+    my $_ranges = $self->_ranges;
+    for my $idx (0 .. $#$_ranges)
+    {
+        my $r = $_ranges->[$idx];
+
+        $r->_flush_and_report;
+        if ($r->is_active && $r->_increment_check_count($MAX_CHECKS))
+        {
+            $r->_guard('');
+            $self->_start_connection($idx);
+        }
+    }
 
     my $time = AnyEvent->now;
     my $last_time = $self->_last_timer_time;
 
     printf "Downloaded %i%% (Currently: %.2fKB/s)\r",
-        int($self->_bytes_dled * 100 / $self->_len),
+        int($total_downloaded * 100 / $self->_len),
         ($num_dloaded / (1024 * ($time-$last_time))),
     ;
     STDOUT->flush;
 
     $self->_last_timer_time($time);
-    $self->_bytes_dled_last_timer($self->_bytes_dled);
 
     return;
 }
@@ -292,8 +314,7 @@ sub run
     {
         my $record = decode_json(_slurp($self->_resume_info_path));
         $self->_len($record->{_len});
-        $self->_bytes_dled($record->{_bytes_dled});
-        $self->_bytes_dled_last_timer($self->_bytes_dled());
+        $self->_downloaded->_my_init($record->{_bytes_dled});
         $self->_remaining_connections($record->{_remaining_connections});
         my $ranges_ref = $record->{_ranges};
         $self->_init_from_len(
@@ -353,7 +374,7 @@ App::ManiacDownloader - a maniac download accelerator.
 
 =head1 VERSION
 
-version 0.0.8
+version 0.0.9
 
 =head1 SYNOPSIS
 
